//http://speleoastronomy.org/elektro/atmega/usingusb.html
#include <windows.h>
#include <setupapi.h>
#include <string>
#include <vector>
#include <map>

//#pragma warning(disable : 4996)

typedef void (WINAPI* t_HidD_GetHidGuid)( OUT LPGUID );
typedef BOOLEAN (WINAPI* t_HidD_GetManufacturerString)(IN HANDLE, OUT PVOID, IN ULONG);
typedef BOOLEAN (WINAPI* t_HidD_GetProductString)(IN HANDLE, OUT PVOID, IN ULONG);
typedef BOOLEAN (WINAPI* t_HidD_GetInputReport)(IN HANDLE, OUT PVOID, IN ULONG);
typedef BOOLEAN (WINAPI* t_HidD_SetOutputReport)(IN HANDLE, IN PVOID, IN ULONG);

using namespace std;

template <typename T> class HIDLibrary
{
protected:
	static const int datasize = sizeof(T);
	T data;
	char pad[16];
	GUID m_hidguid;
	HINSTANCE hDLL;
 public:	t_HidD_GetHidGuid HidD_GetHidGuid;
	t_HidD_GetManufacturerString HidD_GetManufacturerString;
	t_HidD_GetProductString HidD_GetProductString;
	t_HidD_GetInputReport HidD_GetInputReport;
	t_HidD_SetOutputReport HidD_SetOutputReport;

	vector<string> m_HIDPaths;
	map<string,string> m_HIDDeviceIdents;
	string GetDevicePath(HDEVINFO hInfoSet, PSP_DEVICE_INTERFACE_DATA oInterface);


	HIDLibrary();
	bool IsAvailable();
	int EnumerateHIDDevices();
	bool Connect(unsigned int device=0);
	string GetConnectedDevicePath(){return m_ConnectedDevice;};
	string GetConnectedDeviceName(){return m_HIDDeviceIdents[m_ConnectedDevice];};
	string GetDeviceCount() { return m_HIDPaths.size(); };
	string GetDeviceName(int i){ if(i<m_HIDPaths.size())
					  return m_HIDDeviceIdents[m_HIDPaths[i]];
					  else
					  return string.empty;
                                                  }

	int SendData(T* data,HANDLE h);
	int ReceiveData(T* data, HANDLE h);
};

template<  typename T > HIDLibrary< T >::HIDLibrary()//
{
		hDLL = LoadLibrary("HID.DLL");
		if(hDLL != NULL)
		{
		   HidD_GetHidGuid = (t_HidD_GetHidGuid)GetProcAddress(hDLL, "HidD_GetHidGuid");
		   HidD_GetManufacturerString = (t_HidD_GetManufacturerString)GetProcAddress(hDLL, "HidD_GetManufacturerString");
		   HidD_GetProductString = (t_HidD_GetProductString) GetProcAddress(hDLL, "HidD_GetProductString");
		   HidD_GetInputReport = (t_HidD_GetInputReport) GetProcAddress(hDLL, "HidD_GetInputReport");
		   HidD_SetOutputReport = (t_HidD_SetOutputReport) GetProcAddress(hDLL, "HidD_SetOutputReport");


		   if(HidD_GetHidGuid)
		   {
			  HidD_GetHidGuid(&m_hidguid);
		   }
		}
}

template< typename T > bool HIDLibrary< T >::IsAvailable()
{
	return	!(hDLL == NULL || HidD_GetHidGuid == NULL || HidD_GetManufacturerString == NULL ||
			HidD_GetProductString == NULL || HidD_GetInputReport == NULL ||
                         HidD_SetOutputReport == NULL);
}

template<typename T> string HIDLibrary<T>::GetDevicePath(HDEVINFO hInfoSet, PSP_DEVICE_INTERFACE_DATA oInterface)
{
	    char vpath [1000];
		vpath[0]=0;
		DWORD nRequiredSize = 0;
		// Get the device interface details
		if (!SetupDiGetDeviceInterfaceDetail(hInfoSet, oInterface, 0, 0, &nRequiredSize, 0))


		{
			PSP_DEVICE_INTERFACE_DETAIL_DATA oDetail = (PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc(nRequiredSize);
			oDetail->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
                
			if (SetupDiGetDeviceInterfaceDetail(hInfoSet, oInterface, oDetail, nRequiredSize, &nRequiredSize, 0))
			{
				string s(oDetail->DevicePath);

				free(oDetail);

				return s;
			} //ShowMessage("Last err");
		}


		return "";                  
}
//---------------------------
const char idstring[]="vid_aa03&pid_0007";//"VID_AA03&PID_0001";
//---------------------------
template<typename T> int HIDLibrary<T>::EnumerateHIDDevices()
{
     AnsiString test;

		wchar_t buffer[1000];
		char s1[1000];
		m_HIDPaths.clear();
		int nIndex = 0;
		HDEVINFO hInfoSet = SetupDiGetClassDevs(&m_hidguid,0,0,DIGCF_DEVICEINTERFACE | DIGCF_PRESENT);
		SP_DEVICE_INTERFACE_DATA spd;
		spd.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);
        while (SetupDiEnumDeviceInterfaces(hInfoSet, 0, &m_hidguid, nIndex, &spd))
		{
            string path = GetDevicePath(hInfoSet, &spd);
                        
			if(path.find(idstring)!= string::npos)
			{
				HANDLE h = CreateFile(path.c_str(),GENERIC_READ,0,0,OPEN_EXISTING,FILE_FLAG_OVERLAPPED,0);
				if(h != INVALID_HANDLE_VALUE)
				{
					HidD_GetManufacturerString(h,buffer,500);
					wcstombs(s1,buffer, 1000);


					if(strlen(s1)>2)
					{
						strcat(s1," ");
						HidD_GetProductString(h,buffer,500);
 						wcstombs(s1+strlen(s1),buffer, 1000);
					}
					CloseHandle(h);
				;}
				else
  					strcpy(s1,"Unknown device");
				string name(s1);

				m_HIDDeviceIdents[path]=s1;
				m_HIDPaths.push_back(path);
			}
			nIndex++;
		}
		SetupDiDestroyDeviceInfoList(hInfoSet);
		return nIndex;
}

template<typename T> bool HIDLibrary<T>::Connect(unsigned int device)
{
	if(m_HIDPaths.size() < device+1)
	{
		EnumerateHIDDevices();
		if(m_HIDPaths.size() < device+1) return false;
	}
	m_ConnectedDevice = m_HIDPaths[device];
	return true;
}

template<typename T> int HIDLibrary<T>::SendData(T* data,HANDLE hs)
{  unsigned long a;int err;

rep:   int len=65;
   *data=0;
   if(hs != INVALID_HANDLE_VALUE)
   {
    err = HidD_SetOutputReport(hs,data  , len);
          //return 0;//err;
   }

                //if (err==0) {cout<<"sending error"<<endl;
                //goto rep;}

	    return 0;


            }

template<typename T> int HIDLibrary<T>::ReceiveData(T* data,HANDLE hr)
   {


   int len=65;int  err=0;
         if(hr != INVALID_HANDLE_VALUE)
   {                   *data=0;
       	      	 err = HidD_GetInputReport(hr, data , len);


   }
    // if(err==0)  {cout<<endl<<"A recieveing error occured !!!! "<<err<<endl; }

                  
		return 0;
}


